@@ -31,6 +31,7 @@ import org.apache.calcite.linq4j.tree.Expressions;
 import org.apache.calcite.linq4j.tree.ParameterExpression;
 import org.apache.calcite.linq4j.tree.Primitive;
 import org.apache.calcite.linq4j.tree.Statement;
+import org.apache.calcite.linq4j.tree.Types;
 import org.apache.calcite.rel.type.RelDataType;
 import org.apache.calcite.rel.type.RelDataTypeField;
 import org.apache.calcite.rex.RexBuilder;
@@ -45,6 +46,7 @@ import org.apache.calcite.rex.RexLambdaRef;
 import org.apache.calcite.rex.RexLiteral;
 import org.apache.calcite.rex.RexLocalRef;
 import org.apache.calcite.rex.RexNode;
+import org.apache.calcite.rex.RexNodeAndFieldIndex;
 import org.apache.calcite.rex.RexOver;
 import org.apache.calcite.rex.RexPatternFieldRef;
 import org.apache.calcite.rex.RexProgram;
@@ -363,6 +365,7 @@ public class RexToLixTranslator implements RexVisitor<RexToLixTranslator.Result>
 
     switch (targetType.getSqlTypeName()) {
     case ARRAY:
+    case MULTISET:
       final RelDataType sourceDataType = sourceType.getComponentType();
       final RelDataType targetDataType = targetType.getComponentType();
       assert sourceDataType != null;
@@ -585,6 +588,32 @@ public class RexToLixTranslator implements RexVisitor<RexToLixTranslator.Result>
       }
       return defaultExpression.get();
     }
+    case UBIGINT:
+    case UINTEGER:
+    case UTINYINT:
+    case USMALLINT: {
+      Method method;
+      switch (targetType.getSqlTypeName()) {
+      case UBIGINT:
+        method = BuiltInMethod.CAST_TO_ULONG.method;
+        break;
+      case UINTEGER:
+        method = BuiltInMethod.CAST_TO_UINTEGER.method;
+        break;
+      case USMALLINT:
+        method = BuiltInMethod.CAST_TO_USHORT.method;
+        break;
+      case UTINYINT:
+        method = BuiltInMethod.CAST_TO_UBYTE.method;
+        break;
+      default:
+        throw new RuntimeException("Unexpected unsigned type " + targetType.getSqlTypeName());
+      }
+      return Expressions.call(
+          method,
+          operand,
+          Expressions.constant(typeFactory.getTypeSystem().roundingMode()));
+    }
     case BIGINT:
     case INTEGER:
     case TINYINT:
@@ -1614,29 +1643,35 @@ public class RexToLixTranslator implements RexVisitor<RexToLixTranslator.Result>
     if (rexWithStorageTypeResultMap.containsKey(key)) {
       return rexWithStorageTypeResultMap.get(key);
     }
-    final Type storageType = currentStorageType != null
-        ? currentStorageType : typeFactory.getJavaClass(dynamicParam.getType());
+    final Type valueType = typeFactory.getJavaClass(dynamicParam.getType());
+    final Type storageType = currentStorageType != null ? currentStorageType : valueType;
 
     final boolean isNumeric = SqlTypeFamily.NUMERIC.contains(dynamicParam.getType());
 
     // For numeric types, use java.lang.Number to prevent cast exception
     // when the parameter type differs from the target type
-    final Expression valueExpression = isNumeric
+    final Expression rawValueExpression = isNumeric
         ? EnumUtils.convert(
             EnumUtils.convert(
                 Expressions.call(root, BuiltInMethod.DATA_CONTEXT_GET.method,
                     Expressions.constant("?" + dynamicParam.getIndex())),
                 java.lang.Number.class),
-            storageType)
+            valueType)
         : EnumUtils.convert(
             Expressions.call(root, BuiltInMethod.DATA_CONTEXT_GET.method,
                 Expressions.constant("?" + dynamicParam.getIndex())),
-            storageType);
+            valueType);
+
+    final Expression valueExpression =
+        checkExpressionPadTruncate(rawValueExpression,
+            typeFactory.createJavaType(Types.toClass(rawValueExpression.getType())),
+            dynamicParam.getType());
 
     final ParameterExpression valueVariable =
-        Expressions.parameter(valueExpression.getType(),
-            list.newName("value_dynamic_param"));
-    list.add(Expressions.declare(Modifier.FINAL, valueVariable, valueExpression));
+        Expressions.parameter(storageType, list.newName("value_dynamic_param"));
+    list.add(
+        Expressions.declare(Modifier.FINAL, valueVariable,
+            EnumUtils.convert(valueExpression, storageType)));
     final ParameterExpression isNullVariable =
         Expressions.parameter(Boolean.TYPE, list.newName("isNull_dynamic_param"));
     list.add(
@@ -1757,6 +1792,10 @@ public class RexToLixTranslator implements RexVisitor<RexToLixTranslator.Result>
     return new Result(isNullVariable, valueVariable);
   }
 
+  @Override public Result visitNodeAndFieldIndex(RexNodeAndFieldIndex nodeAndFieldIndex) {
+    throw new RuntimeException("cannot translate expression " + nodeAndFieldIndex);
+  }
+
   Expression checkNull(Expression expr) {
     if (Primitive.flavor(expr.getType())
         == Primitive.Flavor.PRIMITIVE) {
