@@ -47,12 +47,15 @@ import org.apache.calcite.schema.Table;
 import org.apache.calcite.schema.TemporalTable;
 import org.apache.calcite.schema.TranslatableTable;
 import org.apache.calcite.schema.Wrapper;
+import org.apache.calcite.schema.lookup.LikePattern;
+import org.apache.calcite.schema.lookup.Lookup;
 import org.apache.calcite.sql.SqlAccessType;
 import org.apache.calcite.sql.validate.SqlModality;
 import org.apache.calcite.sql.validate.SqlMonotonicity;
 import org.apache.calcite.sql2rel.InitializerExpressionFactory;
 import org.apache.calcite.sql2rel.NullInitializerExpressionFactory;
 import org.apache.calcite.util.ImmutableBitSet;
+import org.apache.calcite.util.LazyReference;
 import org.apache.calcite.util.Pair;
 import org.apache.calcite.util.Util;
 
@@ -429,6 +432,8 @@ public class RelOptTableImpl extends Prepare.AbstractPreparingTable {
     private final @Nullable SchemaPlus parent;
     private final String name;
     private final Schema schema;
+    private final LazyReference<Lookup<? extends SchemaPlus>> subSchemas = new LazyReference<>();
+
 
     MySchemaPlus(@Nullable SchemaPlus parent, String name, Schema schema) {
       this.parent = parent;
@@ -455,9 +460,8 @@ public class RelOptTableImpl extends Prepare.AbstractPreparingTable {
       return name;
     }
 
-    @Override public @Nullable SchemaPlus getSubSchema(String name) {
-      final Schema subSchema = schema.getSubSchema(name);
-      return subSchema == null ? null : new MySchemaPlus(this, name, subSchema);
+    @Deprecated @Override public @Nullable SchemaPlus getSubSchema(String name) {
+      return subSchemas().get(name);
     }
 
     @Override public SchemaPlus add(String name, Schema schema) {
@@ -505,12 +509,21 @@ public class RelOptTableImpl extends Prepare.AbstractPreparingTable {
       return false;
     }
 
-    @Override public @Nullable Table getTable(String name) {
-      return schema.getTable(name);
+    @Override public Lookup<Table> tables() {
+      return schema.tables();
     }
 
-    @Override public Set<String> getTableNames() {
-      return schema.getTableNames();
+    @Override public Lookup<? extends SchemaPlus> subSchemas() {
+      return subSchemas.getOrCompute(
+          () -> schema.subSchemas().map((s, key) -> new MySchemaPlus(this, key, s)));
+    }
+
+    @Deprecated @Override public @Nullable Table getTable(String name) {
+      return tables().get(name);
+    }
+
+    @Deprecated @Override public Set<String> getTableNames() {
+      return schema.tables().getNames(LikePattern.any());
     }
 
     @Override public @Nullable RelProtoDataType getType(String name) {
@@ -530,8 +543,8 @@ public class RelOptTableImpl extends Prepare.AbstractPreparingTable {
       return schema.getFunctionNames();
     }
 
-    @Override public Set<String> getSubSchemaNames() {
-      return schema.getSubSchemaNames();
+    @Deprecated @Override public Set<String> getSubSchemaNames() {
+      return schema.subSchemas().getNames(LikePattern.any());
     }
 
     @Override public Expression getExpression(@Nullable SchemaPlus parentSchema,
