@@ -298,6 +298,10 @@ public abstract class Aggregate extends SingleRel implements Hintable {
     return groupSet.cardinality();
   }
 
+  public boolean hasEmptyGroup() {
+    return groupSets.contains(ImmutableBitSet.of());
+  }
+
   /**
    * Returns the number of indicator fields.
    *
@@ -578,8 +582,10 @@ public abstract class Aggregate extends SingleRel implements Hintable {
   public static class AggCallBinding extends SqlOperatorBinding {
     private final List<RelDataType> preOperands;
     private final List<RelDataType> operands;
+    @Deprecated // to be removed before 2.0
     private final int groupCount;
     private final boolean filter;
+    private final boolean hasEmptyGroup;
 
     /**
      * Creates an AggCallBinding.
@@ -590,7 +596,11 @@ public abstract class Aggregate extends SingleRel implements Hintable {
      * @param operands     Data types of operands
      * @param groupCount   Number of columns in the GROUP BY clause
      * @param filter       Whether the aggregate function has a FILTER clause
+     *
+     * @deprecated Use
+     * {@link #AggCallBinding(RelDataTypeFactory, SqlAggFunction, List, List, boolean, boolean)}
      */
+    @Deprecated // to be removed before 2.0
     public AggCallBinding(RelDataTypeFactory typeFactory,
         SqlAggFunction aggFunction, List<RelDataType> preOperands,
         List<RelDataType> operands, int groupCount,
@@ -601,12 +611,37 @@ public abstract class Aggregate extends SingleRel implements Hintable {
           requireNonNull(operands,
               "operands of aggregate call should not be null");
       this.groupCount = groupCount;
+      this.hasEmptyGroup = groupCount == 0;
       this.filter = filter;
       checkArgument(groupCount >= 0,
           "number of group by columns should be greater than zero in "
               + "aggregate call. Got %s", groupCount);
     }
 
+    /**
+     * Creates an AggCallBinding.
+     *
+     * @param typeFactory   Type factory
+     * @param aggFunction   Aggregate function
+     * @param preOperands   Data types of pre-operands
+     * @param operands      Data types of operands
+     * @param hasEmptyGroup Whether the aggregate has a empty group
+     * @param filter        Whether the aggregate function has a FILTER clause
+     */
+    public AggCallBinding(RelDataTypeFactory typeFactory,
+        SqlAggFunction aggFunction, List<RelDataType> preOperands,
+        List<RelDataType> operands, boolean hasEmptyGroup,
+        boolean filter) {
+      super(typeFactory, aggFunction);
+      this.preOperands = requireNonNull(preOperands, "preOperands");
+      this.operands =
+          requireNonNull(operands,
+              "operands of aggregate call should not be null");
+      this.filter = filter;
+      this.hasEmptyGroup = hasEmptyGroup;
+      this.groupCount = hasEmptyGroup ? 0 : 1;
+    }
+
     @Deprecated // to be removed before 2.0
     public AggCallBinding(RelDataTypeFactory typeFactory,
         SqlAggFunction aggFunction, List<RelDataType> operands, int groupCount,
@@ -615,10 +650,15 @@ public abstract class Aggregate extends SingleRel implements Hintable {
           filter);
     }
 
+    @Deprecated // to be removed before 2.0
     @Override public int getGroupCount() {
       return groupCount;
     }
 
+    @Override public boolean hasEmptyGroup() {
+      return hasEmptyGroup;
+    }
+
     @Override public boolean hasFilter() {
       return filter;
     }
@@ -648,9 +688,9 @@ public abstract class Aggregate extends SingleRel implements Hintable {
     private final RelDataType collationType;
 
     PercentileDiscAggCallBinding(RelDataTypeFactory typeFactory, SqlAggFunction aggFunction,
-        List<RelDataType> operands, RelDataType collationType, int groupCount,
+        List<RelDataType> operands, RelDataType collationType, boolean hasEmptyGroup,
         boolean filter) {
-      super(typeFactory, aggFunction, operands, groupCount, filter);
+      super(typeFactory, aggFunction, ImmutableList.of(), operands, hasEmptyGroup, filter);
       assert aggFunction.isPercentile();
       this.collationType = collationType;
     }
