@@ -193,7 +193,8 @@ public class AggregateCall {
         distinctKeys, collation, groupCount, input, type, name);
   }
 
-    /** Creates an AggregateCall, inferring its type if {@code type} is null. */
+  /** Creates an AggregateCall, inferring its type if {@code type} is null. */
+  @Deprecated // to be removed before 2.0
   public static AggregateCall create(SqlAggFunction aggFunction,
       boolean distinct, boolean approximate, boolean ignoreNulls,
       List<RexNode> rexList, List<Integer> argList, int filterArg,
@@ -205,6 +206,7 @@ public class AggregateCall {
         input, type, name);
   }
 
+  @Deprecated // to be removed before 2.0
   public static AggregateCall create(SqlParserPos pos, SqlAggFunction aggFunction,
       boolean distinct, boolean approximate, boolean ignoreNulls,
       List<RexNode> rexList, List<Integer> argList, int filterArg,
@@ -226,6 +228,49 @@ public class AggregateCall {
         rexList, argList, filterArg, distinctKeys, collation, type, name);
   }
 
+  /** Creates an AggregateCall, inferring its type if {@code type} is null. */
+  public static AggregateCall create(SqlAggFunction aggFunction,
+      boolean distinct, boolean approximate, boolean ignoreNulls,
+      List<RexNode> rexList, List<Integer> argList, int filterArg,
+      @Nullable ImmutableBitSet distinctKeys, RelCollation collation,
+      boolean hasEmptyGroup,
+      RelNode input, @Nullable RelDataType type, @Nullable String name) {
+    return create(SqlParserPos.ZERO, aggFunction, distinct, approximate,
+        ignoreNulls, rexList, argList, filterArg, distinctKeys, collation, hasEmptyGroup,
+        input, type, name);
+  }
+
+  public static AggregateCall create(SqlParserPos pos, SqlAggFunction aggFunction,
+      boolean distinct, boolean approximate, boolean ignoreNulls,
+      List<RexNode> rexList, List<Integer> argList, int filterArg,
+      @Nullable ImmutableBitSet distinctKeys, RelCollation collation,
+      boolean hasEmptyGroup,
+      RelNode input, @Nullable RelDataType type, @Nullable String name) {
+    if (type == null) {
+      final RelDataTypeFactory typeFactory =
+          input.getCluster().getTypeFactory();
+      final List<RelDataType> preTypes = RexUtil.types(rexList);
+      final List<RelDataType> types =
+          SqlTypeUtil.projectTypes(input.getRowType(), argList);
+      final Aggregate.AggCallBinding callBinding;
+      if (aggFunction.getKind() == SqlKind.PERCENTILE_DISC
+          || aggFunction.getKind() == SqlKind.PERCENTILE_CONT) {
+        callBinding =
+            new Aggregate.PercentileDiscAggCallBinding(typeFactory, aggFunction,
+                SqlTypeUtil.projectTypes(input.getRowType(), argList),
+                SqlTypeUtil.projectTypes(input.getRowType(), collation.getKeys()).get(0),
+                hasEmptyGroup, filterArg >= 0);
+      } else {
+        callBinding =
+            new Aggregate.AggCallBinding(typeFactory, aggFunction, preTypes, types,
+                hasEmptyGroup, filterArg >= 0);
+      }
+      type = aggFunction.inferReturnType(callBinding);
+    }
+    return create(pos, aggFunction, distinct, approximate, ignoreNulls,
+        rexList, argList, filterArg, distinctKeys, collation, type, name);
+  }
+
   @Deprecated // to be removed before 2.0
   public static AggregateCall create(SqlAggFunction aggFunction,
       boolean distinct, List<Integer> argList, int filterArg, RelDataType type,
@@ -518,11 +563,11 @@ public class AggregateCall {
       return new Aggregate.PercentileDiscAggCallBinding(typeFactory,
           aggFunction, SqlTypeUtil.projectTypes(rowType, argList),
           SqlTypeUtil.projectTypes(rowType, collation.getKeys()).get(0),
-          aggregateRelBase.getGroupCount(), hasFilter());
+          aggregateRelBase.hasEmptyGroup(), hasFilter());
     }
     return new Aggregate.AggCallBinding(typeFactory, aggFunction,
         RexUtil.types(rexList), SqlTypeUtil.projectTypes(rowType, argList),
-        aggregateRelBase.getGroupCount(), hasFilter());
+        aggregateRelBase.hasEmptyGroup(), hasFilter());
   }
 
   /**
@@ -573,6 +618,7 @@ public class AggregateCall {
    * @param newGroupKeyCount number of columns in GROUP BY of new aggregate
    * @return AggregateCall that suits new inputs and GROUP BY columns
    */
+  @Deprecated // to be removed before 2.0
   public AggregateCall adaptTo(RelNode input, List<Integer> argList,
       int filterArg, int oldGroupKeyCount, int newGroupKeyCount) {
     // The return type of aggregate call need to be recomputed.
@@ -588,6 +634,32 @@ public class AggregateCall {
         newGroupKeyCount, input, newType, getName());
   }
 
+  /**
+   * Creates an equivalent AggregateCall that is adapted to a new input types
+   * and/or number of columns in GROUP BY.
+   *
+   * @param input            Relation that will be input of Aggregate
+   * @param argList          Argument indices of the new call in the input
+   * @param filterArg        Index of the filter, or -1
+   * @param oldHasEmptyGroup Whether old aggregate contains empty group
+   * @param newHasEmptyGroup Whether new aggregate contains empty group
+   * @return AggregateCall that suits new inputs and GROUP BY columns
+   */
+  public AggregateCall adaptTo(RelNode input, List<Integer> argList,
+      int filterArg, boolean oldHasEmptyGroup, boolean newHasEmptyGroup) {
+    // The return type of aggregate call need to be recomputed.
+    // Since it might depend on the number of columns in GROUP BY.
+    final RelDataType newType =
+        oldHasEmptyGroup == newHasEmptyGroup
+            && argList.equals(this.argList)
+            && filterArg == this.filterArg
+            ? type
+            : null;
+    return create(pos, aggFunction, distinct, approximate, ignoreNulls,
+        rexList, argList, filterArg, distinctKeys, collation,
+        newHasEmptyGroup, input, newType, getName());
+  }
+
   /** Creates a copy of this aggregate call, applying a mapping to its
    * arguments. */
   public AggregateCall transform(Mappings.TargetMapping mapping) {
