@@ -41,6 +41,7 @@ public class RelWriterImpl implements RelWriter {
   protected final PrintWriter pw;
   protected final SqlExplainLevel detailLevel;
   protected final boolean withIdPrefix;
+  protected final boolean expand;
   protected final Spacer spacer = new Spacer();
   private final List<Pair<String, @Nullable Object>> values = new ArrayList<>();
 
@@ -53,9 +54,15 @@ public class RelWriterImpl implements RelWriter {
   public RelWriterImpl(
       PrintWriter pw, SqlExplainLevel detailLevel,
       boolean withIdPrefix) {
+    this(pw, detailLevel, withIdPrefix, false);
+  }
+  public RelWriterImpl(
+      PrintWriter pw, SqlExplainLevel detailLevel,
+      boolean withIdPrefix, boolean expand) {
     this.pw = pw;
     this.detailLevel = detailLevel;
     this.withIdPrefix = withIdPrefix;
+    this.expand = expand;
   }
 
   //~ Methods ----------------------------------------------------------------
@@ -180,4 +187,8 @@ public class RelWriterImpl implements RelWriter {
     buf.append(")");
     return buf.toString();
   }
+
+  @Override public boolean expand() {
+    return this.expand;
+  }
 }
