@@ -70,11 +70,12 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.NavigableSet;
-import java.util.Objects;
 import java.util.SortedSet;
 import java.util.TreeSet;
 import java.util.stream.Collectors;
 
+import static java.util.Objects.requireNonNull;
+
 /**
  * RelMdCollation supplies a default implementation of
  * {@link org.apache.calcite.rel.metadata.RelMetadataQuery#collations}
@@ -246,7 +247,7 @@ public class RelMdCollation
   public @Nullable ImmutableList<RelCollation> collations(RelSubset rel,
       RelMetadataQuery mq) {
     return copyOf(
-        Objects.requireNonNull(
+        requireNonNull(
             rel.getTraitSet().getTraits(RelCollationTraitDef.INSTANCE)));
   }
 
@@ -316,22 +317,32 @@ public class RelMdCollation
         targetsWithMonotonicity.put(project.i, call.getOperator().getMonotonicity(binding));
       }
     }
-    final List<RelFieldCollation> fieldCollations = new ArrayList<>();
+    List<List<RelFieldCollation>> fieldCollationsList = new ArrayList<>();
   loop:
     for (RelCollation ic : inputCollations) {
       if (ic.getFieldCollations().isEmpty()) {
         continue;
       }
-      fieldCollations.clear();
+      fieldCollationsList.clear();
+      fieldCollationsList.add(new ArrayList<>());
       for (RelFieldCollation ifc : ic.getFieldCollations()) {
         final Collection<Integer> integers = targets.get(ifc.getFieldIndex());
         if (integers.isEmpty()) {
           continue loop; // cannot do this collation
         }
-        fieldCollations.add(ifc.withFieldIndex(integers.iterator().next()));
+        fieldCollationsList = fieldCollationsList.stream()
+            .flatMap(fieldCollations -> integers.stream()
+                .map(integer -> {
+                  List<RelFieldCollation> newFieldCollations = new ArrayList<>(fieldCollations);
+                  newFieldCollations.add(ifc.withFieldIndex(integer));
+                  return newFieldCollations;
+                })).collect(Collectors.toList());
+      }
+      assert !fieldCollationsList.isEmpty();
+      for (List<RelFieldCollation> fieldCollations : fieldCollationsList) {
+        assert !fieldCollations.isEmpty();
+        collations.add(RelCollations.of(fieldCollations));
       }
-      assert !fieldCollations.isEmpty();
-      collations.add(RelCollations.of(fieldCollations));
     }
 
     final List<RelFieldCollation> fieldCollationsForRexCalls =
@@ -562,6 +573,8 @@ public class RelMdCollation
     case ANTI:
     case INNER:
     case LEFT:
+    case ASOF:
+    case LEFT_ASOF:
       return leftCollations;
     case RIGHT:
     case FULL:
