@@ -32,12 +32,15 @@ import org.apache.calcite.rel.core.Sort;
 import org.apache.calcite.rel.core.TableFunctionScan;
 import org.apache.calcite.rel.core.TableModify;
 import org.apache.calcite.rel.core.TableScan;
+import org.apache.calcite.rex.RexCorrelVariable;
+import org.apache.calcite.rex.RexFieldAccess;
 import org.apache.calcite.rex.RexInputRef;
 import org.apache.calcite.rex.RexLocalRef;
 import org.apache.calcite.rex.RexNode;
 import org.apache.calcite.rex.RexShuttle;
 import org.apache.calcite.rex.RexVisitor;
 import org.apache.calcite.rex.RexVisitorImpl;
+import org.apache.calcite.util.Util;
 
 import org.checkerframework.checker.nullness.qual.Nullable;
 import org.checkerframework.checker.nullness.qual.PolyNull;
@@ -84,9 +87,10 @@ public class RelMdColumnOrigins
       Set<RelColumnOrigin> inputSet =
           mq.getColumnOrigins(rel.getInput(), iInput);
       inputSet = createDerivedColumnOrigins(inputSet);
-      if (inputSet != null) {
-        set.addAll(inputSet);
+      if (inputSet == null) {
+        return null;
       }
+      set.addAll(inputSet);
     }
     return set;
   }
@@ -290,7 +294,7 @@ public class RelMdColumnOrigins
     return set;
   }
 
-  private static Set<RelColumnOrigin> getMultipleColumns(RexNode rexNode, RelNode input,
+  private static @Nullable Set<RelColumnOrigin> getMultipleColumns(RexNode rexNode, RelNode input,
       final RelMetadataQuery mq) {
     final Set<RelColumnOrigin> set = new HashSet<>();
     final RexVisitor<Void> visitor =
@@ -303,8 +307,19 @@ public class RelMdColumnOrigins
             }
             return null;
           }
+
+          @Override public Void visitFieldAccess(RexFieldAccess fieldAccess) {
+            if (fieldAccess.getReferenceExpr() instanceof RexCorrelVariable) {
+              throw Util.FoundOne.NULL;
+            }
+            return null;
+          }
         };
-    rexNode.accept(visitor);
+    try {
+      rexNode.accept(visitor);
+    } catch (Util.FoundOne one) {
+      return null;
+    }
     return set;
   }
 }
