@@ -124,9 +124,6 @@ import java.util.stream.Stream;
 import static com.google.common.base.Preconditions.checkArgument;
 
 import static org.apache.calcite.rex.RexLiteral.stringValue;
-import static org.apache.calcite.sql.SqlKind.EXISTS;
-import static org.apache.calcite.sql.SqlKind.IN;
-import static org.apache.calcite.sql.SqlKind.NOT;
 import static org.apache.calcite.util.Util.last;
 
 import static java.util.Objects.requireNonNull;
@@ -478,9 +475,7 @@ public class RelToSqlConverter extends SqlImplementor
       return builder.result();
     } else {
       Result x = visitInput(e, 0, Clause.WHERE);
-      if (e.getCondition().getKind() == NOT
-          || e.getCondition().getKind() == EXISTS
-          || e.getCondition().getKind() == IN) {
+      if (!e.getVariablesSet().isEmpty()) {
         x = x.resetAlias();
       }
       parseCorrelTable(e, x);
@@ -531,7 +526,8 @@ public class RelToSqlConverter extends SqlImplementor
     }
     parseCorrelTable(e, x);
     final Builder builder = x.builder(e);
-    if (!isStar(e.getProjects(), e.getInput().getRowType(), e.getRowType())) {
+    if (!isStar(e.getProjects(), e.getInput().getRowType(), e.getRowType())
+        || !dialect.supportGenerateSelectStar(e.getInput())) {
       final List<SqlNode> selectList = new ArrayList<>();
       for (RexNode ref : e.getProjects()) {
         SqlNode sqlExpr = builder.context.toSql(null, ref);
@@ -1125,9 +1121,6 @@ public class RelToSqlConverter extends SqlImplementor
 
   /** Visits a TableModify; called by {@link #dispatch} via reflection. */
   public Result visit(TableModify modify) {
-    final Map<String, RelDataType> pairs = ImmutableMap.of();
-    final Context context = aliasContext(pairs, false);
-
     // Target Table Name
     final SqlIdentifier sqlTargetTable = getSqlTargetTable(modify);
 
@@ -1146,6 +1139,10 @@ public class RelToSqlConverter extends SqlImplementor
     }
     case UPDATE: {
       final Result input = visitInput(modify, 0);
+      // SELECT statement has two parts: columns from the target table (old values) and
+      // expressions for the UPDATE. See the TableModify documentation for details.
+      final SqlSelect select = input.asSelect();
+      final Context context = selectListContext(select.getSelectList(), false);
 
       final SqlUpdate sqlUpdate =
           new SqlUpdate(POS, sqlTargetTable,
@@ -1155,8 +1152,7 @@ public class RelToSqlConverter extends SqlImplementor
               exprList(context,
                   requireNonNull(modify.getSourceExpressionList(),
                       () -> "modify.getSourceExpressionList() is null for " + modify)),
-              ((SqlSelect) input.node).getWhere(), input.asSelect(),
-              null);
+              select.getWhere(), select, null);
 
       return result(sqlUpdate, input.clauses, modify, null);
     }
@@ -1176,7 +1172,7 @@ public class RelToSqlConverter extends SqlImplementor
       // `WHEN NOT MATCHED THEN INSERT` clauses, the selectList consists of three parts:
       // the insert expression, the target table reference, and the update expression.
       // When querying with the `WHEN MATCHED THEN UPDATE` clause, the selectList will not
-      // include the update expression.
+      // include the insert expression.
       // However, when querying with the `WHEN NOT MATCHED THEN INSERT` clause,
       // the expression list will only contain the insert expression.
       final SqlNodeList selectList = SqlUtil.stripListAs(select.getSelectList());
