@@ -152,7 +152,7 @@ public class AggregateStarTableRule
       }
       for (AggregateCall aggCall : aggregate.getAggCallList()) {
         final AggregateCall copy =
-            rollUp(groupSet.cardinality(), relBuilder, aggCall, tileKey);
+            rollUp(groupSet.isEmpty(), relBuilder, aggCall, tileKey);
         if (copy == null) {
           return;
         }
@@ -194,7 +194,7 @@ public class AggregateStarTableRule
     call.transformTo(relBuilder.build());
   }
 
-  private static @Nullable AggregateCall rollUp(int groupCount,
+  private static @Nullable AggregateCall rollUp(boolean hasEmptyGroup,
       RelBuilder relBuilder, AggregateCall call, TileKey tileKey) {
     if (call.isDistinct()) {
       return null;
@@ -217,7 +217,7 @@ public class AggregateStarTableRule
       return AggregateCall.create(call.getParserPosition(), roll, false, call.isApproximate(),
           call.ignoreNulls(), call.rexList, ImmutableList.of(offset + i), -1,
           call.distinctKeys, call.collation,
-          groupCount, relBuilder.peek(), null, call.name);
+          hasEmptyGroup, relBuilder.peek(), null, call.name);
     }
 
     // Second, try to satisfy the aggregation based on group set columns.
@@ -234,7 +234,7 @@ public class AggregateStarTableRule
       return AggregateCall.create(call.getParserPosition(), aggregation, false,
           call.isApproximate(), call.ignoreNulls(), call.rexList,
           newArgs, -1, call.distinctKeys, call.collation,
-          groupCount, relBuilder.peek(), null, call.name);
+          hasEmptyGroup, relBuilder.peek(), null, call.name);
     }
 
     // No roll up possible.
