@@ -40,6 +40,7 @@ import org.checkerframework.checker.nullness.qual.Nullable;
 import org.immutables.value.Value;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -289,7 +290,7 @@ public class JoinToMultiJoinRule
             null,
             null);
       } else {
-        joinSpecs.add(Pair.of(JoinRelType.INNER, (@Nullable RexNode) null));
+        joinSpecs.add(Pair.of(JoinRelType.INNER, null));
       }
       joinSpecs.add(Pair.of(joinType, joinRel.getCondition()));
       break;
@@ -303,7 +304,7 @@ public class JoinToMultiJoinRule
             right.getRowType().getFieldList(),
             joinRel.getRowType().getFieldList());
       } else {
-        joinSpecs.add(Pair.of(JoinRelType.INNER, (RexNode) null));
+        joinSpecs.add(Pair.of(JoinRelType.INNER, null));
       }
       break;
     default:
@@ -315,7 +316,7 @@ public class JoinToMultiJoinRule
             null,
             null);
       } else {
-        joinSpecs.add(Pair.of(JoinRelType.INNER, (RexNode) null));
+        joinSpecs.add(Pair.of(JoinRelType.INNER, null));
       }
       if (rightCombined) {
         copyOuterJoinInfo(
@@ -325,7 +326,7 @@ public class JoinToMultiJoinRule
             right.getRowType().getFieldList(),
             joinRel.getRowType().getFieldList());
       } else {
-        joinSpecs.add(Pair.of(JoinRelType.INNER, (RexNode) null));
+        joinSpecs.add(Pair.of(JoinRelType.INNER, null));
       }
     }
   }
@@ -358,15 +359,11 @@ public class JoinToMultiJoinRule
     if (adjustmentAmount == 0) {
       destJoinSpecs.addAll(srcJoinSpecs);
     } else {
-      assert srcFields != null;
-      assert destFields != null;
-      int nFields = srcFields.size();
-      int[] adjustments = new int[nFields];
-      for (int idx = 0; idx < nFields; idx++) {
-        adjustments[idx] = adjustmentAmount;
-      }
-      for (Pair<JoinRelType, @Nullable RexNode> src
-          : srcJoinSpecs) {
+      requireNonNull(srcFields, "srcFields");
+      requireNonNull(destFields, "destFields");
+      int[] adjustments = new int[srcFields.size()];
+      Arrays.fill(adjustments, adjustmentAmount);
+      for (Pair<JoinRelType, @Nullable RexNode> src : srcJoinSpecs) {
         destJoinSpecs.add(
             Pair.of(
                 src.left,
@@ -457,9 +454,7 @@ public class JoinToMultiJoinRule
     int nFieldsOnLeft = left.getRowType().getFieldList().size();
     int nFieldsOnRight = right.getRowType().getFieldList().size();
     int[] adjustments = new int[nFieldsOnRight];
-    for (int i = 0; i < nFieldsOnRight; i++) {
-      adjustments[i] = nFieldsOnLeft;
-    }
+    Arrays.fill(adjustments, nFieldsOnLeft);
     rightFilter =
         rightFilter.accept(
             new RelOptUtil.RexInputConverter(
