@@ -44,6 +44,8 @@ import org.apache.calcite.rex.RexDynamicParam;
 import org.apache.calcite.rex.RexExecutor;
 import org.apache.calcite.rex.RexFieldAccess;
 import org.apache.calcite.rex.RexInputRef;
+import org.apache.calcite.rex.RexLambda;
+import org.apache.calcite.rex.RexLambdaRef;
 import org.apache.calcite.rex.RexLiteral;
 import org.apache.calcite.rex.RexLocalRef;
 import org.apache.calcite.rex.RexNode;
@@ -62,7 +64,6 @@ import org.apache.calcite.rex.RexWindowBound;
 import org.apache.calcite.sql.SqlAggFunction;
 import org.apache.calcite.sql.SqlKind;
 import org.apache.calcite.sql.SqlOperator;
-import org.apache.calcite.sql.fun.SqlRowOperator;
 import org.apache.calcite.sql.type.SqlTypeName;
 import org.apache.calcite.tools.RelBuilder;
 import org.apache.calcite.tools.RelBuilderFactory;
@@ -1145,13 +1146,6 @@ public abstract class ReduceExpressionsRule<C extends ReduceExpressionsRule.Conf
         callConstancy = Constancy.NON_CONSTANT;
       }
 
-      // Row operator itself can't be reduced to a literal, but if
-      // the operands are constants, we still want to reduce those
-      if ((callConstancy == Constancy.REDUCIBLE_CONSTANT)
-          && (call.getOperator() instanceof SqlRowOperator)) {
-        callConstancy = Constancy.NON_CONSTANT;
-      }
-
       if (callConstancy == Constancy.NON_CONSTANT) {
         // any REDUCIBLE_CONSTANT children are now known to be maximal
         // reducible subtrees, so they can be added to the result
@@ -1185,6 +1179,14 @@ public abstract class ReduceExpressionsRule<C extends ReduceExpressionsRule.Conf
     @Override public Void visitFieldAccess(RexFieldAccess fieldAccess) {
       return pushVariable();
     }
+
+    @Override public Void visitLambda(RexLambda lambda) {
+      return pushVariable();
+    }
+
+    @Override public Void visitLambdaRef(RexLambdaRef lambda) {
+      return pushVariable();
+    }
   }
 
   /** Shuttle that pushes predicates into a CASE. */
