@@ -55,16 +55,26 @@ public class RexFieldAccess extends RexNode {
 
   private final RexNode expr;
   private final RelDataTypeField field;
+  // Not always the same as the field.getType().
+  private final RelDataType type;
 
   //~ Constructors -----------------------------------------------------------
 
   RexFieldAccess(
       RexNode expr,
       RelDataTypeField field) {
+    this(expr, field, field.getType());
+  }
+
+  RexFieldAccess(
+      RexNode expr,
+      RelDataTypeField field,
+      RelDataType type) {
     checkValid(expr, field);
     this.expr = expr;
     this.field = field;
     this.digest = expr + "." + field.getName();
+    this.type = type;
   }
 
   //~ Methods ----------------------------------------------------------------
@@ -82,7 +92,7 @@ public class RexFieldAccess extends RexNode {
   }
 
   @Override public RelDataType getType() {
-    return field.getType();
+    return type;
   }
 
   @Override public SqlKind getKind() {
