@@ -54,7 +54,9 @@ import java.util.ResourceBundle;
 import java.util.concurrent.Callable;
 import java.util.concurrent.ConcurrentHashMap;
 
-import static java.lang.reflect.Modifier.isStatic;
+import static java.lang.Boolean.parseBoolean;
+
+import static java.lang.Double.parseDouble;
 
 import static org.apache.calcite.linq4j.Nullness.castNonNull;
 
@@ -245,7 +247,9 @@ public class Resources {
         }
         try {
           Inst inst = (Inst) method.invoke(o, args);
-          assert inst != null : "got null from " + method;
+          if (inst == null) {
+            throw new AssertionError("got null from " + method);
+          }
           inst.validate(validations);
         } catch (IllegalAccessException e) {
           throw new RuntimeException("in " + method, e);
@@ -694,7 +698,7 @@ public class Resources {
       super(accessor, method);
       final Default resource = getDefault();
       if (resource != null) {
-        defaultValue = Boolean.parseBoolean(resource.value());
+        defaultValue = parseBoolean(resource.value());
       } else {
         defaultValue = false;
       }
@@ -725,7 +729,7 @@ public class Resources {
       super(accessor, method);
       final Default resource = getDefault();
       if (resource != null) {
-        defaultValue = Double.parseDouble(resource.value());
+        defaultValue = parseDouble(resource.value());
       } else {
         defaultValue = 0d;
       }
@@ -1149,7 +1153,7 @@ public class Resources {
     public boolean booleanValue(BooleanProp p) {
       final String s = properties.getProperty(p.key);
       if (s != null) {
-        return Boolean.parseBoolean(s);
+        return parseBoolean(s);
       }
       p.checkDefault2();
       return p.defaultValue;
@@ -1158,14 +1162,14 @@ public class Resources {
     @Override
     public boolean booleanValue(BooleanProp p, boolean defaultValue) {
       final String s = properties.getProperty(p.key);
-      return s == null ? defaultValue : Boolean.parseBoolean(s);
+      return s == null ? defaultValue : parseBoolean(s);
     }
 
     @Override
     public double doubleValue(DoubleProp p) {
       final String s = properties.getProperty(p.key);
       if (s != null) {
-        return Double.parseDouble(s);
+        return parseDouble(s);
       }
       p.checkDefault2();
       return p.defaultValue;
@@ -1174,7 +1178,7 @@ public class Resources {
     @Override
     public double doubleValue(DoubleProp p, double defaultValue) {
       final String s = properties.getProperty(p.key);
-      return s == null ? defaultValue : Double.parseDouble(s);
+      return s == null ? defaultValue : parseDouble(s);
     }
   }
 }
