@@ -95,6 +95,7 @@ public class SqlCallBinding extends SqlOperatorBinding {
 
   //~ Methods ----------------------------------------------------------------
 
+  @Deprecated
   @Override public int getGroupCount() {
     final SelectScope selectScope =
         SqlValidatorUtil.getEnclosingSelectScope(scope);
@@ -117,6 +118,21 @@ public class SqlCallBinding extends SqlOperatorBinding {
     return validator.isAggregate(select) ? 0 : -1;
   }
 
+  @Override public boolean hasEmptyGroup() {
+    final SelectScope selectScope =
+        SqlValidatorUtil.getEnclosingSelectScope(scope);
+    if (selectScope == null) {
+      // Probably "VALUES expr". Treat same as "SELECT expr GROUP BY ()"
+      return true;
+    }
+    final SqlSelect select = selectScope.getNode();
+    final SqlNodeList group = select.getGroup();
+    if (group != null) {
+      return SqlValidatorUtil.hasEmptyGroup(group);
+    }
+    return validator.isAggregate(select);
+  }
+
   /**
    * Returns the validator.
    */
@@ -175,7 +191,7 @@ public class SqlCallBinding extends SqlOperatorBinding {
 
   /** Returns the operands to a call permuted into the same order as the
    * formal parameters of the function. */
-  private List<SqlNode> permutedOperands(final SqlCall call) {
+  private static List<SqlNode> permutedOperands(final SqlCall call) {
     final SqlOperator operator = call.getOperator();
     final SqlOperandMetadata operandMetadata =
         requireNonNull((SqlOperandMetadata) operator.getOperandTypeChecker(),
