@@ -58,8 +58,28 @@ import static java.util.Objects.requireNonNull;
  */
 public class SqlCollectionTypeNameSpec extends SqlTypeNameSpec {
   private final SqlTypeNameSpec elementTypeName;
+  private final boolean elementTypeNullable;
   private final SqlTypeName collectionTypeName;
 
+  /**
+   * Creates a {@code SqlCollectionTypeNameSpec}.
+   *
+   * @param elementTypeName    Type of the collection element
+   * @param elementTypeNullable Type of the collection element is nullable
+   * @param collectionTypeName Collection type name
+   * @param pos                Parser position, must not be null
+   */
+  public SqlCollectionTypeNameSpec(SqlTypeNameSpec elementTypeName,
+      boolean elementTypeNullable,
+      SqlTypeName collectionTypeName,
+      SqlParserPos pos) {
+    super(new SqlIdentifier(collectionTypeName.name(), pos), pos);
+    this.elementTypeName = requireNonNull(elementTypeName, "elementTypeName");
+    this.elementTypeNullable = elementTypeNullable;
+    this.collectionTypeName =
+        requireNonNull(collectionTypeName, "collectionTypeName");
+  }
+
   /**
    * Creates a {@code SqlCollectionTypeNameSpec}.
    *
@@ -70,10 +90,7 @@ public class SqlCollectionTypeNameSpec extends SqlTypeNameSpec {
   public SqlCollectionTypeNameSpec(SqlTypeNameSpec elementTypeName,
       SqlTypeName collectionTypeName,
       SqlParserPos pos) {
-    super(new SqlIdentifier(collectionTypeName.name(), pos), pos);
-    this.elementTypeName = requireNonNull(elementTypeName, "elementTypeName");
-    this.collectionTypeName =
-        requireNonNull(collectionTypeName, "collectionTypeName");
+    this(elementTypeName, true, collectionTypeName, pos);
   }
 
   public SqlTypeNameSpec getElementTypeName() {
@@ -82,8 +99,9 @@ public class SqlCollectionTypeNameSpec extends SqlTypeNameSpec {
 
   @Override public RelDataType deriveType(SqlValidator validator) {
     RelDataType type = elementTypeName.deriveType(validator);
-    // We have to assume that elements may be nullable
-    type = validator.getTypeFactory().enforceTypeWithNullability(type, true);
+    if (elementTypeNullable) {
+      type = validator.getTypeFactory().enforceTypeWithNullability(type, true);
+    }
     return createCollectionType(type, validator.getTypeFactory());
   }
 
