@@ -23,6 +23,7 @@ import org.apache.calcite.config.CharLiteralStyle;
 import org.apache.calcite.config.NullCollation;
 import org.apache.calcite.linq4j.function.Experimental;
 import org.apache.calcite.rel.RelFieldCollation;
+import org.apache.calcite.rel.RelNode;
 import org.apache.calcite.rel.core.JoinRelType;
 import org.apache.calcite.rel.type.RelDataType;
 import org.apache.calcite.rel.type.RelDataTypeSystem;
@@ -754,6 +755,10 @@ public class SqlDialect {
     return true;
   }
 
+  public boolean supportsOrderByLiteral() {
+    return true;
+  }
+
   public boolean supportsAggregateFunction(SqlKind kind) {
     switch (kind) {
     case LITERAL_AGG:
@@ -797,6 +802,11 @@ public class SqlDialect {
     return true;
   }
 
+  /** Returns whether this dialect supports generate 'SELECT *'. */
+  public boolean supportGenerateSelectStar(RelNode relNode) {
+    return true;
+  }
+
   /** Converts {@link RexNode} expression to {@link RexNode} expression before unparse. */
   public RexNode prepareUnparse(RexNode rexNode) {
     return rexNode;
@@ -1311,6 +1321,8 @@ public class SqlDialect {
       return SqlConformanceEnum.ORACLE_10;
     case MSSQL:
       return SqlConformanceEnum.SQL_SERVER_2008;
+    case PRESTO:
+      return SqlConformanceEnum.PRESTO;
     default:
       return SqlConformanceEnum.PRAGMATIC_2003;
     }
