@@ -18,6 +18,9 @@ package org.apache.calcite.sql.dialect;
 
 import org.apache.calcite.avatica.util.Casing;
 import org.apache.calcite.avatica.util.TimeUnitRange;
+import org.apache.calcite.rel.RelNode;
+import org.apache.calcite.rel.core.Join;
+import org.apache.calcite.rel.rules.MultiJoin;
 import org.apache.calcite.rel.type.RelDataType;
 import org.apache.calcite.rel.type.RelDataTypeSystem;
 import org.apache.calcite.rel.type.RelDataTypeSystemImpl;
@@ -45,13 +48,16 @@ import org.apache.calcite.sql.fun.SqlStdOperatorTable;
 import org.apache.calcite.sql.parser.SqlParserPos;
 import org.apache.calcite.sql.type.SqlTypeName;
 import org.apache.calcite.sql.type.SqlTypeUtil;
+import org.apache.calcite.util.Util;
 
 import com.google.common.collect.ImmutableList;
 
 import org.checkerframework.checker.nullness.qual.Nullable;
 
 import java.util.List;
+import java.util.Locale;
 import java.util.Objects;
+import java.util.stream.Collectors;
 
 import static org.apache.calcite.linq4j.Nullness.castNonNull;
 
@@ -179,6 +185,19 @@ public class PostgresqlSqlDialect extends SqlDialect {
     return false;
   }
 
+  @Override public boolean supportGenerateSelectStar(RelNode relNode) {
+    // Whether the relNode is a join and whether its inputs have duplicate field names.
+    // For example, EMP JOIN DEPT, both of which have columns named DEPTNO.
+    if (relNode instanceof Join || relNode instanceof MultiJoin) {
+      final List<String> fieldNames = relNode.getInputs().stream()
+          .flatMap(input -> input.getRowType().getFieldNames().stream())
+          .map(name -> isCaseSensitive() ? name : name.toLowerCase(Locale.ROOT))
+          .collect(Collectors.toList());
+      return Util.isDistinct(fieldNames);
+    }
+    return true;
+  }
+
   @Override public void unparseCall(SqlWriter writer, SqlCall call,
       int leftPrec, int rightPrec) {
     switch (call.getKind()) {
@@ -214,6 +233,10 @@ public class PostgresqlSqlDialect extends SqlDialect {
     return false;
   }
 
+  @Override public boolean supportsOrderByLiteral() {
+    return false;
+  }
+
   @Override public void unparseSqlSetOption(SqlWriter writer,
       int leftPrec, int rightPrec, SqlSetOption option) {
     String scope = option.getScope();
