@@ -25,16 +25,15 @@ import org.apache.calcite.schema.CustomColumnResolvingTable;
 import org.apache.calcite.schema.Table;
 import org.apache.calcite.sql.SqlCall;
 import org.apache.calcite.sql.SqlIdentifier;
+import org.apache.calcite.sql.SqlLambda;
 import org.apache.calcite.sql.SqlNode;
 import org.apache.calcite.sql.SqlNodeList;
 import org.apache.calcite.sql.SqlSelect;
 import org.apache.calcite.sql.SqlWindow;
 import org.apache.calcite.sql.parser.SqlParserPos;
-import org.apache.calcite.sql.type.SqlTypeName;
 import org.apache.calcite.util.Pair;
 import org.apache.calcite.util.Util;
 
-import com.google.common.base.Preconditions;
 import com.google.common.base.Suppliers;
 import com.google.common.collect.ImmutableList;
 
@@ -49,6 +48,9 @@ import java.util.List;
 import java.util.Map;
 import java.util.function.Supplier;
 
+import static com.google.common.base.Preconditions.checkArgument;
+
+import static org.apache.calcite.sql.validate.SqlNonNullableAccessors.getSelectList;
 import static org.apache.calcite.util.Static.RESOURCE;
 
 import static java.util.Objects.requireNonNull;
@@ -233,6 +235,8 @@ public abstract class DelegatingScope implements SqlValidatorScope {
   @Override public SqlValidatorScope getOperandScope(SqlCall call) {
     if (call instanceof SqlSelect) {
       return validator.getSelectScope((SqlSelect) call);
+    } else if (call instanceof SqlLambda) {
+      return validator.getLambdaScope((SqlLambda) call);
     }
     return this;
   }
@@ -422,7 +426,7 @@ public abstract class DelegatingScope implements SqlValidatorScope {
         }
       }
       if (requireNonNull(fromPath, "fromPath").stepCount() > 1) {
-        assert fromRowType != null;
+        requireNonNull(fromRowType, "fromRowType");
         for (Step p : fromPath.steps()) {
           fromRowType = fromRowType.getFieldList().get(p.i).getType();
         }
@@ -587,6 +591,7 @@ public abstract class DelegatingScope implements SqlValidatorScope {
           }
         }
       }
+      return false;
     } else { // check if there are fields with the same name
       int count = 0;
       for (RelDataTypeField f : rowType.getFieldList()) {
@@ -594,16 +599,12 @@ public abstract class DelegatingScope implements SqlValidatorScope {
           count++;
         }
       }
-      if (count > 1) {
-        return true;
-      }
+      return count > 1;
     }
-    return false;
   }
 
   private AggregatingSelectScope.Resolved resolve() {
-    Preconditions.checkArgument(groupAnalyzer == null,
-        "resolve already in progress");
+    checkArgument(groupAnalyzer == null, "resolve already in progress");
     SqlValidatorUtil.GroupAnalyzer groupAnalyzer = new SqlValidatorUtil.GroupAnalyzer();
     this.groupAnalyzer = groupAnalyzer;
     try {
@@ -636,7 +637,7 @@ public abstract class DelegatingScope implements SqlValidatorScope {
               });
         } else {
           rowType.getFieldList().forEach(field -> {
-            if (field.getType().getSqlTypeName() == SqlTypeName.MEASURE) {
+            if (field.getType().isMeasure()) {
               analyzer.measureExprs.add(
                   new SqlIdentifier(
                       Arrays.asList(child.name, field.getName()),
@@ -654,4 +655,45 @@ public abstract class DelegatingScope implements SqlValidatorScope {
   public SqlValidatorScope getParent() {
     return parent;
   }
+
+  /** Qualifies an identifier by looking for an alias in the current
+   * select-list.
+   *
+   * <p>Used when resolving ORDER BY items (when the conformance allows order by
+   * alias, such as "SELECT x - y AS z FROM t ORDER BY z") and measures
+   * (when one measure refers to another, for example
+   * "SELECT SUM(x) AS MEASURE m1, SUM(y) - m1 AS MEASURE m2 FROM t"). */
+  protected @Nullable SqlQualified qualifyUsingAlias(SqlSelect select,
+      SqlIdentifier identifier) {
+    final String name = identifier.names.get(0);
+    final SqlNameMatcher nameMatcher = validator.catalogReader.nameMatcher();
+    final int aliasCount = aliasCount(select, nameMatcher, name);
+    switch (aliasCount) {
+    case 0:
+      return null;
+    case 1:
+      final SqlValidatorNamespace selectNs =
+          validator.getNamespaceOrThrow(select);
+      return SqlQualified.create(this, 1, selectNs, identifier);
+    default:
+      // More than one column has this alias.
+      throw validator.newValidationError(identifier,
+          RESOURCE.columnAmbiguous(name));
+    }
+  }
+
+  /** Returns the number of columns in the SELECT clause that have {@code name}
+   * as their implicit (e.g. {@code t.name}) or explicit (e.g.
+   * {@code t.c as name}) alias. */
+  private static int aliasCount(SqlSelect select, SqlNameMatcher nameMatcher,
+      String name) {
+    int n = 0;
+    for (SqlNode s : getSelectList(select)) {
+      final @Nullable String alias = SqlValidatorUtil.alias(s);
+      if (alias != null && nameMatcher.matches(alias, name)) {
+        n++;
+      }
+    }
+    return n;
+  }
 }
