@@ -130,6 +130,11 @@ public interface SqlConformance {
    */
   SelectAliasLookup isSelectAlias();
 
+  /**
+   * Allow non-GROUP BY columns in SELECT (MySQL loose mode).
+   */
+  boolean isNonStrictGroupBy();
+
   /**
    * Whether {@code GROUP BY 2} is interpreted to mean 'group by the 2nd column
    * in the select list'.
@@ -527,6 +532,7 @@ public interface SqlConformance {
    * {@link SqlConformanceEnum#ORACLE_12},
    * {@link SqlConformanceEnum#PRESTO},
    * {@link SqlConformanceEnum#SQL_SERVER_2008};
+   * {@link SqlConformanceEnum#BIG_QUERY}
    * false otherwise.
    */
   boolean shouldConvertRaggedUnionTypesToVarying();
@@ -657,4 +663,9 @@ public interface SqlConformance {
    * they terminate with a fatal error on overflow.
    */
   boolean checkedArithmetic();
+
+  /**
+   * True when the unsigned versions of integer types are supported.
+   */
+  boolean supportsUnsignedTypes();
 }
