@@ -506,4 +506,25 @@ public enum SqlConformanceEnum implements SqlConformance {
       return true;
     }
   }
+
+  @Override public boolean supportsUnsignedTypes() {
+    switch (this) {
+    case BABEL:
+    case DEFAULT:
+    case LENIENT:
+    case MYSQL_5:
+      return true;
+    default:
+      return false;
+    }
+  }
+
+  @Override public boolean isNonStrictGroupBy() {
+    switch (this) {
+    case BABEL:
+      return true;
+    default:
+      return false;
+    }
+  }
 }
