@@ -51,6 +51,10 @@ public class SqlDelegatingConformance implements SqlConformance {
     return delegate.isSelectAlias();
   }
 
+  @Override public boolean isNonStrictGroupBy() {
+    return delegate.isNonStrictGroupBy();
+  }
+
   @Override public boolean isGroupByOrdinal() {
     return delegate.isGroupByOrdinal();
   }
@@ -165,4 +169,8 @@ public class SqlDelegatingConformance implements SqlConformance {
   @Override public boolean checkedArithmetic() {
     return delegate.checkedArithmetic();
   }
+
+  @Override public boolean supportsUnsignedTypes() {
+    return delegate.supportsUnsignedTypes();
+  }
 }
