@@ -31,6 +31,8 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.function.BiConsumer;
 
+import static java.util.Objects.requireNonNull;
+
 /**
  * Pair of objects.
  *
@@ -42,7 +44,7 @@ import java.util.function.BiConsumer;
  * @param <T2> Right-hand type
  */
 @SuppressWarnings("type.argument.type.incompatible")
-public class Pair<T1 extends @Nullable Object, T2 extends @Nullable Object>
+public class Pair<T1, T2>
     implements Comparable<Pair<T1, T2>>, Map.Entry<T1, T2>, Serializable {
 
   @SuppressWarnings({"rawtypes", "unchecked"})
@@ -421,7 +423,7 @@ public class Pair<T1 extends @Nullable Object, T2 extends @Nullable Object>
     private final E first;
 
     FirstAndIterator(Iterator<? extends E> iterator, E first) {
-      this.iterator = Objects.requireNonNull(iterator, "iterator");
+      this.iterator = requireNonNull(iterator, "iterator");
       this.first = first;
     }
 
@@ -448,8 +450,8 @@ public class Pair<T1 extends @Nullable Object, T2 extends @Nullable Object>
 
     ZipIterator(Iterator<? extends L> leftIterator,
         Iterator<? extends R> rightIterator) {
-      this.leftIterator = Objects.requireNonNull(leftIterator, "leftIterator");
-      this.rightIterator = Objects.requireNonNull(rightIterator, "rightIterator");
+      this.leftIterator = requireNonNull(leftIterator, "leftIterator");
+      this.rightIterator = requireNonNull(rightIterator, "rightIterator");
     }
 
     @Override public boolean hasNext() {
@@ -471,14 +473,12 @@ public class Pair<T1 extends @Nullable Object, T2 extends @Nullable Object>
    *
    * @param <E> Element type */
   private static class AdjacentIterator<E> implements Iterator<Pair<E, E>> {
-    private final E first;
     private final Iterator<? extends E> iterator;
     E previous;
 
     AdjacentIterator(Iterator<? extends E> iterator) {
-      this.iterator = Objects.requireNonNull(iterator, "iterator");
-      this.first = iterator.next();
-      previous = first;
+      this.iterator = requireNonNull(iterator, "iterator");
+      previous = iterator.next();
     }
 
     @Override public boolean hasNext() {
@@ -539,8 +539,8 @@ public class Pair<T1 extends @Nullable Object, T2 extends @Nullable Object>
     private final List<V> vs;
 
     MutableZipList(List<K> ks, List<V> vs) {
-      this.ks = Objects.requireNonNull(ks, "ks");
-      this.vs = Objects.requireNonNull(vs, "vs");
+      this.ks = requireNonNull(ks, "ks");
+      this.vs = requireNonNull(vs, "vs");
     }
 
     @Override public Pair<K, V> get(int index) {
