@@ -62,6 +62,8 @@ public abstract class RexProgramBuilderBase {
   protected RexLiteral nullSmallInt;
   protected RexLiteral nullVarchar;
   protected RexLiteral nullDecimal;
+  protected RexLiteral nullReal;
+  protected RexLiteral nullDouble;
   protected RexLiteral nullVarbinary;
 
   private RelDataType nullableBool;
@@ -79,6 +81,12 @@ public abstract class RexProgramBuilderBase {
   private RelDataType nullableDecimal;
   private RelDataType nonNullableDecimal;
 
+  private RelDataType nullableReal;
+  private RelDataType nonNullableReal;
+
+  private RelDataType nullableDouble;
+  private RelDataType nonNullableDouble;
+
   private RelDataType nullableVarbinary;
   private RelDataType nonNullableVarbinary;
 
@@ -123,6 +131,14 @@ public abstract class RexProgramBuilderBase {
     nullableDecimal = typeFactory.createTypeWithNullability(nonNullableDecimal, true);
     nullDecimal = rexBuilder.makeNullLiteral(nullableDecimal);
 
+    nonNullableReal = typeFactory.createSqlType(SqlTypeName.REAL);
+    nullableReal = typeFactory.createTypeWithNullability(nonNullableReal, true);
+    nullReal = rexBuilder.makeNullLiteral(nullableReal);
+
+    nonNullableDouble = typeFactory.createSqlType(SqlTypeName.DOUBLE);
+    nullableDouble = typeFactory.createTypeWithNullability(nonNullableDouble, true);
+    nullDouble = rexBuilder.makeNullLiteral(nullableDouble);
+
     nonNullableVarbinary = typeFactory.createSqlType(SqlTypeName.VARBINARY);
     nullableVarbinary = typeFactory.createTypeWithNullability(nonNullableVarbinary, true);
     nullVarbinary = rexBuilder.makeNullLiteral(nullableVarbinary);
@@ -425,13 +441,29 @@ public abstract class RexProgramBuilderBase {
   }
 
   protected RelDataType tDecimal() {
-    return nonNullableDecimal;
+    return tDecimal(false);
   }
 
   protected RelDataType tDecimal(boolean nullable) {
     return nullable ? nullableDecimal : nonNullableDecimal;
   }
 
+  protected RelDataType tReal() {
+    return tReal(false);
+  }
+
+  protected RelDataType tReal(boolean nullable) {
+    return nullable ? nullableReal : nonNullableReal;
+  }
+
+  protected RelDataType tDouble() {
+    return tDouble(false);
+  }
+
+  protected RelDataType tDouble(boolean nullable) {
+    return nullable ? nullableDouble : nonNullableDouble;
+  }
+
   protected RelDataType tBigInt() {
     return tBigInt(false);
   }
@@ -507,6 +539,9 @@ public abstract class RexProgramBuilderBase {
     return rexBuilder.makeLiteral(value, nonNullableVarchar);
   }
 
+  protected RexLiteral literal(double value) {
+    return rexBuilder.makeApproxLiteral(value, nonNullableDouble);
+  }
   // Variables
 
   /**
