@@ -93,6 +93,7 @@ class SqlAdvisorTest extends SqlValidatorTestCase {
           "TABLE(CATALOG.SALES.DEPT)",
           "TABLE(CATALOG.SALES.DEPTNULLABLES)",
           "TABLE(CATALOG.SALES.DEPT_SINGLE)",
+          "TABLE(CATALOG.SALES.DOUBLE_PK)",
           "TABLE(CATALOG.SALES.DEPT_NESTED)",
           "TABLE(CATALOG.SALES.DEPT_NESTED_EXPANDED)",
           "TABLE(CATALOG.SALES.BONUS)",
@@ -299,6 +300,7 @@ class SqlAdvisorTest extends SqlValidatorTestCase {
 
   protected static final List<String> PREDICATE_KEYWORDS =
       Arrays.asList(
+          "KEYWORD(&)",
           "KEYWORD(()",
           "KEYWORD(*)",
           "KEYWORD(+)",
@@ -308,6 +310,7 @@ class SqlAdvisorTest extends SqlValidatorTestCase {
           "KEYWORD(%)",
           "KEYWORD(<)",
           "KEYWORD(<=)",
+          "KEYWORD(<<)",
           "KEYWORD(<>)",
           "KEYWORD(!=)",
           "KEYWORD(=)",
@@ -334,6 +337,7 @@ class SqlAdvisorTest extends SqlValidatorTestCase {
           "KEYWORD(SUBMULTISET)",
           "KEYWORD(SUCCEEDS)",
           "KEYWORD([)",
+          "KEYWORD(^)",
           "KEYWORD(||)");
 
   private static final List<String> WHERE_KEYWORDS =
