@@ -29,6 +29,7 @@ import com.google.common.collect.ImmutableList;
 import org.hamcrest.Matcher;
 import org.junit.jupiter.api.Test;
 
+import java.math.BigInteger;
 import java.util.List;
 import java.util.Locale;
 import java.util.stream.Collectors;
@@ -37,10 +38,13 @@ import static org.apache.calcite.sql.type.SqlTypeUtil.areSameFamily;
 import static org.apache.calcite.sql.type.SqlTypeUtil.convertTypeToSpec;
 import static org.apache.calcite.sql.type.SqlTypeUtil.equalAsCollectionSansNullability;
 import static org.apache.calcite.sql.type.SqlTypeUtil.equalAsMapSansNullability;
+import static org.apache.calcite.sql.type.SqlTypeUtil.integerBound;
+import static org.apache.calcite.sql.type.SqlTypeUtil.integerRangeContains;
 import static org.apache.calcite.test.Matchers.isListOf;
 
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
+import static org.junit.jupiter.api.Assertions.assertThrows;
 
 /**
  * Test of {@link org.apache.calcite.sql.type.SqlTypeUtil}.
@@ -110,6 +114,14 @@ class SqlTypeUtilTest {
         is(false));
   }
 
+  private RelDataType struct(RelDataType...relDataTypes) {
+    final RelDataTypeFactory.Builder builder = f.typeFactory.builder();
+    for (int i = 0; i < relDataTypes.length; i++) {
+      builder.add("field" + i, relDataTypes[i]);
+    }
+    return builder.build();
+  }
+
   @Test void testModifyTypeCoercionMappings() {
     SqlTypeMappingRules.Builder builder = SqlTypeMappingRules.builder();
     final SqlTypeCoercionRule defaultRules = SqlTypeCoercionRule.instance();
@@ -198,20 +210,59 @@ class SqlTypeUtilTest {
     assertThat(fieldTypeNames, isListOf("INTEGER", "INTEGER"));
   }
 
+  /**
+   * Test for <a href="https://issues.apache.org/jira/browse/CALCITE-7186">[CALCITE-7186]</a>
+   * Add mapping from Character[] to VARCHAR in Java UDF.
+   */
+  @Test void testJavaToSqlCharacterArrayAndByteArrayMapping() {
+    SqlTypeName sqlTypeName = JavaToSqlTypeConversionRules.instance().lookup(Character[].class);
+    assertThat("Character[].class should map to SqlTypeName.VARCHAR",
+        sqlTypeName, is(SqlTypeName.VARCHAR));
+  }
+
+  @Test void testIntegerRangeContainsForSignedAndUnsigned() {
+    RelDataType uTinyInt = f.typeFactory.createSqlType(SqlTypeName.UTINYINT);
+    RelDataType tinyInt = f.typeFactory.createSqlType(SqlTypeName.TINYINT);
+    RelDataType smallInt = f.typeFactory.createSqlType(SqlTypeName.SMALLINT);
+    RelDataType intType = f.typeFactory.createSqlType(SqlTypeName.INTEGER);
+
+    assertThat(integerRangeContains(intType, smallInt), is(true));
+    assertThat(integerRangeContains(intType, tinyInt), is(true));
+    assertThat(integerRangeContains(intType, uTinyInt), is(true));
+    assertThat(integerRangeContains(tinyInt, uTinyInt), is(false));
+    assertThrows(IllegalArgumentException.class,
+        () -> integerRangeContains(f.sqlVarchar, intType));
+  }
+
+  @Test void testIntegerRangeContainsWithDecimal() {
+    RelDataType intType = f.typeFactory.createSqlType(SqlTypeName.INTEGER);
+    RelDataType dec9 = f.typeFactory.createSqlType(SqlTypeName.DECIMAL, 9, 0);
+    RelDataType dec10 = f.typeFactory.createSqlType(SqlTypeName.DECIMAL, 10, 0);
+    RelDataType dec10Scale1 = f.typeFactory.createSqlType(SqlTypeName.DECIMAL, 10, 1);
+
+    assertThat(integerRangeContains(intType, dec9), is(true));
+    assertThat(integerRangeContains(intType, dec10), is(false));
+    assertThat(integerRangeContains(intType, dec10Scale1), is(false));
+  }
+
+  @Test void testIntegerBound() {
+    RelDataType intType = f.typeFactory.createSqlType(SqlTypeName.INTEGER);
+    RelDataType uIntType = f.typeFactory.createSqlType(SqlTypeName.UINTEGER);
+    RelDataType dec10 = f.typeFactory.createSqlType(SqlTypeName.DECIMAL, 10, 0);
+    RelDataType dec10Scale1 = f.typeFactory.createSqlType(SqlTypeName.DECIMAL, 10, 1);
+
+    assertThat(integerBound(intType, false), is(BigInteger.valueOf(Integer.MIN_VALUE)));
+    assertThat(integerBound(intType, true), is(BigInteger.valueOf(Integer.MAX_VALUE)));
+    assertThat(integerBound(uIntType, true), is(BigInteger.valueOf((1L << 32) - 1)));
+    assertThat(integerBound(dec10, true), is(BigInteger.valueOf(9_999_999_999L)));
+    assertThat(integerBound(dec10Scale1, true), is((BigInteger) null));
+  }
+
   @Test void testGetMaxPrecisionScaleDecimal() {
     RelDataType decimal = SqlTypeUtil.getMaxPrecisionScaleDecimal(f.typeFactory);
     assertThat(decimal, is(f.typeFactory.createSqlType(SqlTypeName.DECIMAL, 19, 9)));
   }
 
-
-  private RelDataType struct(RelDataType...relDataTypes) {
-    final RelDataTypeFactory.Builder builder = f.typeFactory.builder();
-    for (int i = 0; i < relDataTypes.length; i++) {
-      builder.add("field" + i, relDataTypes[i]);
-    }
-    return builder.build();
-  }
-
   private void compareTypesIgnoringNullability(
       String comment, RelDataType type1, RelDataType type2, boolean expectedResult) {
     String typeString1 = type1.getFullTypeString();
