@@ -19,12 +19,22 @@ package org.apache.calcite.test;
 import org.apache.calcite.config.CalciteConnectionProperty;
 import org.apache.calcite.config.Lex;
 import org.apache.calcite.sql.fun.SqlLibrary;
+import org.apache.calcite.sql.parser.impl.SqlParserImpl;
 import org.apache.calcite.sql.validate.SqlConformanceEnum;
 
+import com.google.common.collect.ImmutableSet;
+
+import net.hydromatic.quidem.Command;
+import net.hydromatic.quidem.CommandHandler;
 import net.hydromatic.quidem.Quidem;
 
+import org.checkerframework.checker.nullness.qual.Nullable;
+
 import java.sql.Connection;
 import java.util.Collection;
+import java.util.List;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 import static org.apache.calcite.util.Util.discard;
 
@@ -186,4 +196,31 @@ class CoreQuidemTest extends QuidemTest {
     }
     checkRun(path);
   }
+
+  @Override protected CommandHandler createCommandHandler() {
+    return new ExtendedCommandHandler();
+  }
+
+  /** Command handler that adds a "!explain-validated-on dialect..." command
+   * (see {@link QuidemTest.ExplainValidatedCommand}). */
+  private static class ExtendedCommandHandler implements CommandHandler {
+    @Override public @Nullable Command parseCommand(List<String> lines,
+        List<String> content, String line) {
+      final String prefix = "explain-validated-on";
+      if (line.startsWith(prefix)) {
+        final Pattern pattern =
+            Pattern.compile("explain-validated-on( [-_+a-zA-Z0-9]+)*?");
+        final Matcher matcher = pattern.matcher(line);
+        if (matcher.matches()) {
+          final ImmutableSet.Builder<String> set = ImmutableSet.builder();
+          for (int i = 0; i < matcher.groupCount(); i++) {
+            set.add(matcher.group(i + 1));
+          }
+          return new QuidemTest.ExplainValidatedCommand(
+              SqlParserImpl.FACTORY, lines, content, set.build());
+        }
+      }
+      return null;
+    }
+  }
 }
