@@ -35,6 +35,7 @@ import org.apache.calcite.test.schemata.hr.Employee;
 import org.apache.calcite.test.schemata.hr.Event;
 import org.apache.calcite.test.schemata.hr.HrSchema;
 import org.apache.calcite.test.schemata.hr.Location;
+import org.apache.calcite.test.schemata.hr.NullableTest;
 import org.apache.calcite.util.JsonBuilder;
 import org.apache.calcite.util.Smalls;
 import org.apache.calcite.util.TryThreadLocal;
@@ -366,7 +367,7 @@ public class MaterializationTest {
    * Implementation of RelVisitor to extract substituted table names.
    */
   private static class TableNameVisitor extends RelVisitor {
-    private List<List<String>> names = new ArrayList<>();
+    private final List<List<String>> names = new ArrayList<>();
 
     List<List<String>> run(RelNode input) {
       go(input);
@@ -416,6 +417,9 @@ public class MaterializationTest {
         new Dependent(10, "Michael"),
         new Dependent(10, "Jane"),
     };
+    public final NullableTest[] nullables = {
+        new NullableTest(null, null, 1),
+    };
     public final Dependent[] locations = {
         new Dependent(10, "San Francisco"),
         new Dependent(20, "San Diego"),
