@@ -850,77 +850,77 @@ class SqlHintsConverterTest {
       }
 
       @Override public RelNode visit(TableScan scan) {
-        if (scan.getHints().size() > 0) {
+        if (!scan.getHints().isEmpty()) {
           this.hintsCollect.add("TableScan:" + scan.getHints());
         }
         return super.visit(scan);
       }
 
       @Override public RelNode visit(LogicalJoin join) {
-        if (join.getHints().size() > 0) {
+        if (!join.getHints().isEmpty()) {
           this.hintsCollect.add("LogicalJoin:" + join.getHints());
         }
         return super.visit(join);
       }
 
       @Override public RelNode visit(LogicalProject project) {
-        if (project.getHints().size() > 0) {
+        if (!project.getHints().isEmpty()) {
           this.hintsCollect.add("Project:" + project.getHints());
         }
         return super.visit(project);
       }
 
       @Override public RelNode visit(LogicalAggregate aggregate) {
-        if (aggregate.getHints().size() > 0) {
+        if (!aggregate.getHints().isEmpty()) {
           this.hintsCollect.add("Aggregate:" + aggregate.getHints());
         }
         return super.visit(aggregate);
       }
 
       @Override public RelNode visit(LogicalCorrelate correlate) {
-        if (correlate.getHints().size() > 0) {
+        if (!correlate.getHints().isEmpty()) {
           this.hintsCollect.add("Correlate:" + correlate.getHints());
         }
         return super.visit(correlate);
       }
 
       @Override public RelNode visit(LogicalFilter filter) {
-        if (filter.getHints().size() > 0) {
+        if (!filter.getHints().isEmpty()) {
           this.hintsCollect.add("Filter:" + filter.getHints());
         }
         return super.visit(filter);
       }
 
       @Override public RelNode visit(LogicalUnion union) {
-        if (union.getHints().size() > 0) {
+        if (!union.getHints().isEmpty()) {
           this.hintsCollect.add("Union:" + union.getHints());
         }
         return super.visit(union);
       }
 
       @Override public RelNode visit(LogicalIntersect intersect) {
-        if (intersect.getHints().size() > 0) {
+        if (!intersect.getHints().isEmpty()) {
           this.hintsCollect.add("Intersect:" + intersect.getHints());
         }
         return super.visit(intersect);
       }
 
       @Override public RelNode visit(LogicalMinus minus) {
-        if (minus.getHints().size() > 0) {
+        if (!minus.getHints().isEmpty()) {
           this.hintsCollect.add("Minus:" + minus.getHints());
         }
         return super.visit(minus);
       }
 
       @Override public RelNode visit(LogicalSort sort) {
-        if (sort.getHints().size() > 0) {
+        if (!sort.getHints().isEmpty()) {
           this.hintsCollect.add("Sort:" + sort.getHints());
         }
         return super.visit(sort);
       }
 
       @Override public RelNode visit(LogicalValues values) {
-        if (values.getHints().size() > 0) {
+        if (!values.getHints().isEmpty()) {
           this.hintsCollect.add("Values:" + values.getHints());
         }
         return super.visit(values);
@@ -929,17 +929,17 @@ class SqlHintsConverterTest {
       @Override public RelNode visit(RelNode other) {
         if (other instanceof Window) {
           Window window = (Window) other;
-          if (window.getHints().size() > 0) {
+          if (!window.getHints().isEmpty()) {
             this.hintsCollect.add("Window:" + window.getHints());
           }
         } else if (other instanceof Snapshot) {
           Snapshot snapshot = (Snapshot) other;
-          if (snapshot.getHints().size() > 0) {
+          if (!snapshot.getHints().isEmpty()) {
             this.hintsCollect.add("Snapshot:" + snapshot.getHints());
           }
         } else if (other instanceof TableFunctionScan) {
           TableFunctionScan scan = (TableFunctionScan) other;
-          if (scan.getHints().size() > 0) {
+          if (!scan.getHints().isEmpty()) {
             this.hintsCollect.add("TableFunctionScan:" + scan.getHints());
           }
         }
