@@ -182,6 +182,18 @@ class UdfTest {
         + Smalls.AllTypesFunction.class.getName()
         + "',\n"
         + "           methodName: '*'\n"
+        + "         },\n"
+        + "         {\n"
+        + "           name: 'UNBASE64',\n"
+        + "           className: '"
+        + Smalls.MyUnbase64Function.class.getName()
+        + "'\n"
+        + "         },\n"
+        + "         {\n"
+        + "           name: 'CHARACTERARRAY',\n"
+        + "           className: '"
+        + Smalls.CharacterArrayFunction.class.getName()
+        + "'\n"
         + "         }\n"
         + "       ]\n"
         + "     }\n"
@@ -728,7 +740,7 @@ class UdfTest {
 
   /** Test case for
    * <a href="https://issues.apache.org/jira/browse/CALCITE-1434">[CALCITE-1434]
-   * AggregateFunctionImpl doesnt work if the class implements a generic
+   * AggregateFunctionImpl doesn't work if the class implements a generic
    * interface</a>. */
   @Test void testUserDefinedAggregateFunctionImplementsInterface() {
     final String empDept = JdbcTest.EmpDeptTableFactory.class.getName();
@@ -1037,11 +1049,9 @@ class UdfTest {
         .returnsValue("0");
   }
 
-  /**
-   * Test case for
+  /** Test case for
    * <a href="https://issues.apache.org/jira/browse/CALCITE-1834">[CALCITE-1834]
-   * User-defined function for Arrays</a>.
-   */
+   * User-defined function for Arrays</a>. */
   @Test void testArrayUserDefinedFunction() throws Exception {
     try (Connection connection = DriverManager.getConnection("jdbc:calcite:")) {
       CalciteConnection calciteConnection =
@@ -1075,6 +1085,38 @@ class UdfTest {
     }
   }
 
+  /** Test case for
+   * <a href="https://issues.apache.org/jira/browse/CALCITE-7073">[CALCITE-7073]
+   * If the UDF's Java return type is ByteString, Calcite should deduce that the
+   * SQL type is VARBINARY</a>.
+   *
+   * <p>Tests that the {@code UNBASE64} user-defined function correctly decodes
+   * a Base64 string and its return type ({@code VARBINARY}, mapped from
+   * {@link org.apache.calcite.avatica.util.ByteString}) is fully compatible for
+   * direct comparison with SQL {@code VARBINARY} literals ({@code X'...'})
+   * within queries. */
+  @Test void testUnbase64DirectComparison() {
+    final String testHex = "74657374"; // "test" in bytes
+    final String testBase64 = "dGVzdA=="; // Base64 for "test"
+
+    final String sql = "values \"adhoc\".unbase64('" + testBase64 + "')";
+    withUdf().query(sql).typeIs("[EXPR$0 VARBINARY]");
+
+    final String sql2 = "select \"adhoc\".unbase64(cast('" + testBase64
+        + "' as varchar)) = x'" + testHex + "' as C\n";
+    withUdf().query(sql2).returns("C=true\n");
+  }
+
+  /**
+   * Test for <a href="https://issues.apache.org/jira/browse/CALCITE-7186">[CALCITE-7186]</a>
+   * Add mapping from Character[] to VARCHAR in Java UDF.
+   */
+  @Test void testCharacterArrayComparison() {
+    final String testString = "abc";
+    String sql = "values \"adhoc\".characterarray('" + testString + "')";
+    withUdf().query(sql).typeIs("[EXPR$0 VARCHAR]");
+  }
+
   /**
    * Base class for functions that append arrays.
    */
