@@ -21,6 +21,8 @@ import org.apache.calcite.util.Util;
 
 import com.google.common.collect.ImmutableList;
 
+import org.checkerframework.checker.nullness.qual.Nullable;
+
 import java.io.PrintStream;
 import java.math.BigDecimal;
 import java.math.BigInteger;
@@ -39,6 +41,8 @@ import java.util.Set;
 import java.util.TreeMap;
 import java.util.regex.Pattern;
 
+import static java.util.Objects.requireNonNull;
+
 /**
  * ConcurrentTestCommandGenerator creates instances of
  * {@link ConcurrentTestCommand} that perform specific actions in a specific
@@ -72,15 +76,15 @@ public class ConcurrentTestCommandGenerator {
 
 
   /**
-   * Maps Integer thread IDs to a TreeMap. The TreeMap vaules map an Integer
+   * Maps Integer thread IDs to a TreeMap. The TreeMap values map an Integer
    * execution order to a {@link ConcurrentTestCommand}.
    */
-  private TreeMap<Integer, TreeMap<Integer, ConcurrentTestCommand>> threadMap;
+  private final Map<Integer, TreeMap<Integer, ConcurrentTestCommand>> threadMap;
 
   /**
    * Maps Integer thread IDs to thread names.
    */
-  private TreeMap<Integer, String> threadNameMap;
+  private final Map<Integer, String> threadNameMap;
 
   /**
    * Describes a thread that failed.
@@ -102,7 +106,7 @@ public class ConcurrentTestCommandGenerator {
    * execution has ended. Only failed threads appear in the list, so after a
    * successful test the list is empty.
    */
-  private List<FailedThread> failedThreads;
+  private final List<FailedThread> failedThreads;
 
   /**
    * Constructs a new ConcurrentTestCommandGenerator.
@@ -171,7 +175,7 @@ public class ConcurrentTestCommandGenerator {
       int threadId,
       int order,
       String sql) {
-    assert sql != null;
+    requireNonNull(sql, "sql");
 
     ConcurrentTestCommand command = new ExplainCommand(sql);
 
@@ -193,7 +197,7 @@ public class ConcurrentTestCommandGenerator {
       int threadId,
       int order,
       String sql) {
-    assert sql != null;
+    requireNonNull(sql, "sql");
 
     ConcurrentTestCommand command = new PrepareCommand(sql);
 
@@ -434,8 +438,7 @@ public class ConcurrentTestCommandGenerator {
     return threads;
   }
 
-  protected void postExecute(ConcurrentTestCommandExecutor[] threads)
-      throws Exception {
+  protected void postExecute(ConcurrentTestCommandExecutor[] threads) {
     // check for failures
     if (requiresCustomErrorHandling()) {
       for (ConcurrentTestCommandExecutor executor : threads) {
@@ -629,8 +632,8 @@ public class ConcurrentTestCommandGenerator {
   protected abstract static class AbstractCommand
       implements ConcurrentTestCommand {
     private boolean shouldFail = false;
-    private String failComment = null; // describes an expected error
-    private Pattern failPattern = null; // an expected error message
+    private @Nullable String failComment = null; // describes an expected error
+    private @Nullable Pattern failPattern = null; // an expected error message
     private boolean failureExpected = false; // failure expected, no pattern
 
     // implement ConcurrentTestCommand
@@ -723,7 +726,7 @@ public class ConcurrentTestCommandGenerator {
    * the test before continuing.
    */
   private static class SleepCommand extends AbstractCommand {
-    private long millis;
+    private final long millis;
 
     private SleepCommand(long millis) {
       this.millis = millis;
@@ -741,7 +744,7 @@ public class ConcurrentTestCommandGenerator {
    * {@link #execute(ConcurrentTestCommandExecutor)}.
    */
   private static class ExplainCommand extends AbstractCommand {
-    private String sql;
+    private final String sql;
 
     private ExplainCommand(String sql) {
       this.sql = sql;
@@ -777,7 +780,7 @@ public class ConcurrentTestCommandGenerator {
    * statement in the ConcurrentTestCommandExecutor.
    */
   private static class PrepareCommand extends AbstractCommand {
-    private String sql;
+    private final String sql;
 
     private PrepareCommand(String sql) {
       this.sql = sql;
@@ -811,7 +814,7 @@ public class ConcurrentTestCommandGenerator {
 
   /** Command that executes statements with a given timeout. */
   private abstract static class CommandWithTimeout extends AbstractCommand {
-    private int timeout;
+    private final int timeout;
 
     private CommandWithTimeout(int timeout) {
       this.timeout = timeout;
@@ -1048,7 +1051,7 @@ public class ConcurrentTestCommandGenerator {
 
       int colNum = 1;
       while (expectedIter.hasNext() && resultIter.hasNext()) {
-        Object expectedValue = expectedIter.next();
+        @Nullable Object expectedValue = expectedIter.next();
         Object resultValue = resultIter.next();
 
         if ((expectedValue == null)
@@ -1112,8 +1115,8 @@ public class ConcurrentTestCommandGenerator {
     }
 
     private void test(
-        Object expected,
-        Object got,
+        @Nullable Object expected,
+        @Nullable Object got,
         int rowNum,
         int colNum) {
       if ((expected == null) && (got == null)) {
@@ -1270,8 +1273,8 @@ public class ConcurrentTestCommandGenerator {
         expectedOut.append(" |");
         resultOut.append(" |");
 
-        fullMessage.append('\n').append(expectedOut.toString()).append(
-            '\n').append(resultOut.toString());
+        fullMessage.append('\n').append(expectedOut)
+            .append('\n').append(resultOut);
 
         rowNum++;
       }
@@ -1285,7 +1288,7 @@ public class ConcurrentTestCommandGenerator {
    * {@link Statement#executeUpdate(String)}.
    */
   private static class InsertCommand extends CommandWithTimeout {
-    private String sql;
+    private final String sql;
 
     private InsertCommand(
         int timeout,
@@ -1333,7 +1336,7 @@ public class ConcurrentTestCommandGenerator {
    * {@link #doExecute(ConcurrentTestCommandExecutor)}.
    */
   private static class DdlCommand extends AbstractCommand {
-    private String sql;
+    private final String sql;
 
     private DdlCommand(String sql) {
       this.sql = sql;
