@@ -237,6 +237,9 @@ public class DruidDateTimeUtils {
       return ranges.build();
 
     case SEARCH:
+      if (!canTransformSearchToRange(call)) {
+        return null;
+      }
       final RexLiteral right = (RexLiteral) call.operands.get(1);
       final Sarg<?> sarg = requireNonNull(right.getValueAs(Sarg.class));
       ranges = ImmutableList.builder();
@@ -255,6 +258,28 @@ public class DruidDateTimeUtils {
     }
   }
 
+  /**
+   * Returns whether the given SEARCH call can be transformed to a Druid range.
+   *
+   * @param call a SEARCH call
+   * @return whether the given SEARCH call can be transformed to a Druid range.
+   */
+  private static boolean canTransformSearchToRange(RexCall call) {
+    assert call.getKind() == SqlKind.SEARCH;
+    if (call.getOperands().get(0) instanceof RexInputRef) {
+      SqlTypeName literalType = call.operands.get(1).getType().getSqlTypeName();
+      switch (literalType) {
+      case TIMESTAMP:
+      case TIMESTAMP_WITH_LOCAL_TIME_ZONE:
+      case DATE:
+        return true;
+      default:
+        return false;
+      }
+    }
+    return false;
+  }
+
   private static Long toLong(Comparable comparable) {
     if (comparable instanceof TimestampString) {
       TimestampString timestampString = (TimestampString) comparable;
