@@ -73,6 +73,10 @@ import java.util.ArrayList;
 import java.util.Deque;
 import java.util.List;
 
+import static com.google.common.base.Preconditions.checkArgument;
+
+import static java.util.Objects.requireNonNull;
+
 /**
  * Visits pig expression plans and converts them into corresponding RexNodes.
  */
@@ -248,42 +252,46 @@ class PigRelExVisitor extends LogicalExpressionVisitor {
     }
   }
 
-  @Override public void visit(NegativeExpression op) throws FrontendException {
+  @Override public void visit(NegativeExpression op) {
     final RexNode operand = stack.pop();
     if (operand instanceof RexLiteral) {
       final Comparable value = ((RexLiteral) operand).getValue();
-      assert value instanceof BigDecimal;
-      stack.push(builder.literal(((BigDecimal) value).negate()));
+      if (value instanceof BigDecimal) {
+        stack.push(builder.literal(((BigDecimal) value).negate()));
+      } else {
+        assert value instanceof Double;
+        stack.push(builder.literal(- (Double) value));
+      }
     } else {
       stack.push(builder.call(SqlStdOperatorTable.UNARY_MINUS, operand));
     }
   }
 
-  @Override public void visit(EqualExpression op) throws FrontendException {
+  @Override public void visit(EqualExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.EQUALS, buildBinaryOperands()));
   }
 
-  @Override public void visit(NotEqualExpression op) throws FrontendException {
+  @Override public void visit(NotEqualExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.NOT_EQUALS, buildBinaryOperands()));
   }
 
-  @Override public void visit(LessThanExpression op) throws FrontendException {
+  @Override public void visit(LessThanExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.LESS_THAN, buildBinaryOperands()));
   }
 
-  @Override public void visit(LessThanEqualExpression op) throws FrontendException {
+  @Override public void visit(LessThanEqualExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.LESS_THAN_OR_EQUAL, buildBinaryOperands()));
   }
 
-  @Override public void visit(GreaterThanExpression op) throws FrontendException {
+  @Override public void visit(GreaterThanExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.GREATER_THAN, buildBinaryOperands()));
   }
 
-  @Override public void visit(GreaterThanEqualExpression op) throws FrontendException {
+  @Override public void visit(GreaterThanEqualExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.GREATER_THAN_OR_EQUAL, buildBinaryOperands()));
   }
 
-  @Override public void visit(RegexExpression op) throws FrontendException {
+  @Override public void visit(RegexExpression op) {
     RexNode operand1 = replacePatternIfPossible(stack.pop());
     RexNode operand2 = replacePatternIfPossible(stack.pop());
     stack.push(builder.call(SqlStdOperatorTable.LIKE, ImmutableList.of(operand2, operand1)));
@@ -302,43 +310,43 @@ class PigRelExVisitor extends LogicalExpressionVisitor {
     return rexNode;
   }
 
-  @Override public void visit(IsNullExpression op) throws FrontendException {
+  @Override public void visit(IsNullExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.IS_NULL, stack.pop()));
   }
 
-  @Override public void visit(NotExpression op) throws FrontendException {
+  @Override public void visit(NotExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.NOT, stack.pop()));
   }
 
-  @Override public void visit(AndExpression op) throws FrontendException {
+  @Override public void visit(AndExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.AND, buildBinaryOperands()));
   }
 
-  @Override public void visit(OrExpression op) throws FrontendException {
+  @Override public void visit(OrExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.OR, buildBinaryOperands()));
   }
 
-  @Override public void visit(AddExpression op) throws FrontendException {
+  @Override public void visit(AddExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.PLUS, buildBinaryOperands()));
   }
 
-  @Override public void visit(SubtractExpression op) throws FrontendException {
+  @Override public void visit(SubtractExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.MINUS, buildBinaryOperands()));
   }
 
-  @Override public void visit(MultiplyExpression op) throws FrontendException {
+  @Override public void visit(MultiplyExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.MULTIPLY, buildBinaryOperands()));
   }
 
-  @Override public void visit(ModExpression op) throws FrontendException {
+  @Override public void visit(ModExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.MOD, buildBinaryOperands()));
   }
 
-  @Override public void visit(DivideExpression op) throws FrontendException {
+  @Override public void visit(DivideExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.DIVIDE, buildBinaryOperands()));
   }
 
-  @Override public void visit(BinCondExpression op) throws FrontendException {
+  @Override public void visit(BinCondExpression op) {
     stack.push(builder.call(SqlStdOperatorTable.CASE, buildOperands(3)));
   }
 
@@ -370,10 +378,11 @@ class PigRelExVisitor extends LogicalExpressionVisitor {
     return list != null ? list.size() : 0;
   }
 
-  @Override public void visit(DereferenceExpression op) throws FrontendException {
+  @Override public void visit(DereferenceExpression op) {
     final RexNode parentField = stack.pop();
     List<Integer> cols = op.getBagColumns();
-    assert cols != null && cols.size() > 0;
+    requireNonNull(cols, "cols");
+    checkArgument(!cols.isEmpty());
 
     if (parentField.getType() instanceof MultisetSqlType) {
       // Calcite does not support projection on Multiset type. We build
@@ -422,13 +431,13 @@ class PigRelExVisitor extends LogicalExpressionVisitor {
     }
   }
 
-  @Override public void visit(MapLookupExpression op) throws FrontendException {
+  @Override public void visit(MapLookupExpression op) {
     final RexNode relKey = builder.literal(op.getLookupKey());
     final RexNode relMap = stack.pop();
     stack.push(builder.call(SqlStdOperatorTable.ITEM, relMap, relKey));
   }
 
-  @Override public void visit(ScalarExpression op) throws FrontendException {
+  @Override public void visit(ScalarExpression op) {
     // First operand is the path to the materialized view
     RexNode operand1 = stack.pop();
     assert operand1 instanceof RexLiteral
