@@ -222,7 +222,7 @@ public abstract class AbstractSqlTester implements SqlTester, AutoCloseable {
   }
 
   @Override public void check(SqlTestFactory factory,
-      String query, TypeChecker typeChecker,
+      String queryWithCarets, TypeChecker typeChecker,
       ParameterChecker parameterChecker, ResultChecker resultChecker) {
     // This implementation does NOT check the result!
     // All it does is check the return type.
@@ -232,12 +232,14 @@ public abstract class AbstractSqlTester implements SqlTester, AutoCloseable {
 
     // Parse and validate. There should be no errors.
     // There must be 1 column. Get its type.
-    RelDataType actualType = getColumnType(factory, query);
+    RelDataType actualType = getColumnType(factory, queryWithCarets);
 
     // Check result type.
-    typeChecker.checkType(() -> "Query: " + query, actualType);
+    typeChecker.checkType(() -> "Query: " + queryWithCarets, actualType);
 
-    Pair<SqlValidator, SqlNode> p = parseAndValidate(factory, query);
+    // Unescape the carets that may be present in the query
+    final StringAndPos pos = StringAndPos.of(queryWithCarets);
+    Pair<SqlValidator, SqlNode> p = parseAndValidate(factory, pos.sql);
     SqlValidator validator = p.left;
     SqlNode n = p.right;
     final RelDataType parameterRowType = validator.getParameterRowType(n);
@@ -265,12 +267,12 @@ public abstract class AbstractSqlTester implements SqlTester, AutoCloseable {
     if (runtime) {
       // We need to test that the expression fails at runtime.
       // Ironically, that means that it must succeed at prepare time.
-      final String sql = buildQuery(sap.addCarets());
+      final String sql = buildQuery(sap);
       Pair<SqlValidator, SqlNode> p = parseAndValidate(factory, sql);
       SqlNode n = p.right;
       assertNotNull(n);
     } else {
-      StringAndPos sap1 = StringAndPos.of(buildQuery(sap.addCarets()));
+      StringAndPos sap1 = StringAndPos.of(buildQuery(sap));
       checkQueryFails(factory, sap1, expectedError);
     }
   }
@@ -296,8 +298,23 @@ public abstract class AbstractSqlTester implements SqlTester, AutoCloseable {
     }
   }
 
-  public static String buildQuery(String expression) {
-    return "values (" + expression + ")";
+  /**
+   * Build a legal query from an expression with carets.
+   *
+   * <p>@deprecated Please use {@link AbstractSqlTester#buildQuery(StringAndPos)}
+   */
+  @Deprecated
+  public static String buildQuery(String expressionWithCarets) {
+    StringAndPos sap = StringAndPos.of(expressionWithCarets);
+    return buildQuery(sap);
+  }
+
+  public static String buildQuery(StringAndPos sap) {
+    return "values (" + sap.sql + ")";
+  }
+
+  public static StringAndPos buildQueryWithPos(StringAndPos sap) {
+    return StringAndPos.of("values (" + sap.addCarets() + ")");
   }
 
   public static String buildQueryAgg(String expression) {
@@ -322,21 +339,24 @@ public abstract class AbstractSqlTester implements SqlTester, AutoCloseable {
    * {@code CASE 1 WHEN 2 THEN 'a' ELSE NULL END} are left as is.
    *
    * @param factory Test factory
-   * @param expression Scalar expression
+   * @param expressionWithCarets Scalar expression that may contain carets
    * @return Query that evaluates a scalar expression
    */
-  protected String buildQuery2(SqlTestFactory factory, String expression) {
-    if (expression.matches("(?i).*(percentile_(cont|disc)|convert|sort_array|cast)\\(.*")) {
+  protected String buildQuery2(SqlTestFactory factory, String expressionWithCarets) {
+    if (expressionWithCarets.matches(
+        "(?i).*(percentile_(cont|disc)|convert|sort_array|cast)\\(.*")) {
       // PERCENTILE_CONT requires its argument to be a literal,
       // so converting its argument to a column will cause false errors.
       // Similarly, MSSQL-style CONVERT.
-      return buildQuery(expression);
+      return buildQuery(expressionWithCarets);
     }
     // "values (1 < 5)"
     // becomes
     // "select p0 < p1 from (values (1, 5)) as t(p0, p1)"
     SqlNode x;
-    final String sql = "values (" + expression + ")";
+    // Expression may contain escaped carets; unescape them.
+    final StringAndPos sap = StringAndPos.of(expressionWithCarets);
+    final String sql = "values (" + sap.sql + ")";
     try {
       x = parseQuery(factory, sql);
     } catch (SqlParseException e) {
@@ -447,12 +467,13 @@ public abstract class AbstractSqlTester implements SqlTester, AutoCloseable {
   }
 
   @Override public void forEachQuery(SqlTestFactory factory,
-      String expression, Consumer<String> consumer) {
+      String expressionWithCarets, Consumer<String> consumer) {
     // Why not return a list? If there is a syntax error in the expression, the
     // consumer will discover it before we try to parse it to do substitutions
     // on the parse tree.
-    consumer.accept("values (" + expression + ")");
-    consumer.accept(buildQuery2(factory, expression));
+    consumer.accept("values (" + expressionWithCarets + ")");
+    final String query2 = buildQuery2(factory, expressionWithCarets);
+    consumer.accept(SqlParserUtil.escapeCarets(query2));
   }
 
   @Override public void assertConvertsTo(SqlTestFactory factory,
