@@ -16,6 +16,8 @@
  */
 package org.apache.calcite.test.schemata.hr;
 
+import org.checkerframework.checker.nullness.qual.Nullable;
+
 import java.sql.Timestamp;
 import java.util.List;
 
@@ -26,7 +28,7 @@ public class DepartmentPlus extends Department {
   public final Timestamp inceptionDate;
 
   public DepartmentPlus(int deptno, String name, List<Employee> employees,
-                        Location location, Timestamp inceptionDate) {
+      @Nullable Location location, Timestamp inceptionDate) {
     super(deptno, name, employees, location);
     this.inceptionDate = inceptionDate;
   }
