@@ -19,6 +19,7 @@ package org.apache.calcite.util;
 import org.apache.calcite.DataContext;
 import org.apache.calcite.adapter.enumerable.EnumerableTableScan;
 import org.apache.calcite.adapter.java.AbstractQueryableTable;
+import org.apache.calcite.avatica.util.ByteString;
 import org.apache.calcite.config.CalciteConnectionConfig;
 import org.apache.calcite.linq4j.AbstractEnumerable;
 import org.apache.calcite.linq4j.BaseQueryable;
@@ -1477,4 +1478,32 @@ public class Smalls {
           BuiltInMethod.QUERYABLE_AS_ENUMERABLE.method);
     }
   }
+
+  /** User-defined function that decodes a Base64 string to bytes. */
+  public static class MyUnbase64Function {
+    public static ByteString eval(String s) {
+      if (s == null) {
+        return null;
+      }
+      try {
+        return ByteString.ofBase64(s);
+      } catch (Exception e) {
+        return null;
+      }
+    }
+  }
+
+  /** User-defined function with return type Character[]. */
+  public static class CharacterArrayFunction {
+    public static Character[] eval(String s) {
+      if (s == null) {
+        return null;
+      }
+      Character[] characters = new Character[s.length()];
+      for (int i = 0; i < s.length(); i++) {
+        characters[i] = s.charAt(i);
+      }
+      return characters;
+    }
+  }
 }
